// @format

const { Command, flags } = require('@oclif/command')
const express = require('express')
const app = express()
const http = require('http').Server(app)
const jsonParser = require('socket.io-json-parser')
const io = require('socket.io')(http, {
	parser: jsonParser,
})
const wd = require('wd')
const mq = require('../lib/mq')
const { runDoctor } = require('../lib/doctor')

const Version = 1.1

// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/AsyncFunction
// var AsyncFunction = Object.getPrototypeOf(async function(){}).constructor
// var f = new AsyncFunction('driver', 'return await driver.source();');
// console.log(await f(driver))

class StartCommand extends Command {
	async run() {
		io.on('connection', async (socket) => {
			let driverOpts = {
				protocol: 'http',
				host: '127.0.0.1',
				port: 4723,
				path: '/wd/hub',
			}
			let driverSingle = null
			let driver = () => {
				if (driverSingle == null) {
					console.log(`Initializing driver with opts:\n${JSON.stringify(driverOpts, null, '\t')}`)
					driverSingle = wd.promiseChainRemote(driverOpts)
				}
				return driverSingle
			}

			console.log('client connected')

			let statusCheck = setInterval(async () => {
				try {
					let cap = await driver().sessionCapabilities()
					socket.emit('status', { appium: cap.udid, xcodeOrgId: cap.xcodeOrgId })
				} catch (err) {
					socket.emit('status', { appium: null, xcodeOrgId: null })
				}
			}, 1000)

			socket.on('call', async (msg) => {
				if (process.env.MQ_ENV === 'dev') {
					console.dir(msg)
				}

				const respond = (type, args) => socket.emit('return', { id: msg.id, type, ...args })

				try {
					switch (msg.type) {
						case 'driverOpts':
							driverOpts = { ...driverOpts, ...msg.opts }
							driverSingle = null
							respond('driverOpts', driverOpts)
							break

						case 'info':
							respond('info', { version: Version })
							break

						case 'appium':
							{
								let state = {
									wd,
									driver: driver(),
									util: {
										async makeObject(...map) {
											let r = {}
											for (let i = 0; i < map.length; i += 2) {
												r[map[i]] =
													map[i + 1] != null && map[i + 1].var != null ? await state[map[i + 1].var] : map[i + 1]
											}
											return r
										},
										async getWidgetOffset(el, x, y) {
											let element = el.var != null ? state[el.var] : el
											let [loc, size] = await Promise.all([element.getLocation(), element.getSize()])
											return {
												x: loc.x + x * size.width,
												y: loc.y + y * size.height,
											}
										},

										async getWindowOffset(x, y) {
											let size = await state.driver.getWindowSize()
											return {
												x: x * size.width,
												y: y * size.height,
											}
										},
									},
								}

								let prev = driver()
								for (let { rec, cmd, args, out, construct } of msg.cmds) {
									let receiver = rec != null ? state[rec] : prev
									let a = await Promise.all(
										(args || []).map(async (arg) => (arg != null && arg.var != null ? await state[arg.var] : arg)),
									)

									prev = construct ? new receiver[cmd](...a) : receiver[cmd](...a)
									if (out != null) {
										state[out] = prev
									}
								}

								respond('appium', { result: await prev })
							}
							break

						case 'launch':
							respond('launch', { result: await mq.launchAndroidPackage(msg.udid, msg.appId, msg.clearCache === true) })
							break

						case 'devices':
							if (msg.appType === 'android') {
								respond('devices', { devices: await mq.getAndroidDevices() })
							} else if (msg.appType === 'ios') {
								respond('devices', { devices: await mq.getiOSDevices() })
							}
							break

						case 'doctor':
							respond('doctor', { results: await runDoctor(msg.appType) })
							break

						default:
							console.error(`Unknown command: ${msg.type}`)
							respond('error', { error: `Unknown command: ${msg.type}` })
							break
					}
				} catch (err) {
					console.error(err)
					respond('error', { error: err.toString() })
				}
			})

			socket.on('disconnect', async () => {
				console.log('client disconnected')

				clearInterval(statusCheck)
			})
		})

		http.listen(4000, function() {
			console.log('listening on *:4000')
		})
	}
}

StartCommand.description = `Starts Barista's Local Server`

StartCommand.flags = {}

module.exports = StartCommand
