// @format

const http = require('http')
const child = require('child_process')
const fs = require('fs')
const path = require('path')

const exec = async (cmd) =>
	await new Promise((resolve, reject) =>
		child.exec(cmd, (err, stdout, stderr) => (err == null ? resolve([stdout, stderr]) : reject(err))),
	)

const stat = async (path) =>
	await new Promise((resolve, reject) => fs.stat(path, (err, stats) => (err == null ? resolve(stats) : reject(err))))

const isAppiumInstalled = async () => {
	try {
		await exec('appium --version')
		return true
	} catch (err) {}

	try {
		await exec('npx appium --version')
		return true
	} catch (err) {}

	return false
}

const isAppiumRunning = async () =>
	await new Promise((resolve, reject) => {
		let req = http.get('http://127.0.0.1:4723/wd/hub/status', (rsp) => {
			const status = parseInt(rsp.statusCode)
			resolve(status >= 200 && status < 400)
		})
		req.on('error', () => resolve(false))
	})

const isAndroidHomeSet = async () => {
	try {
		let [adb, aapt] = await Promise.all([
			stat(path.resolve(process.env['ANDROID_HOME'], 'platform-tools', 'adb')),
			stat(path.resolve(process.env['ANDROID_HOME'], 'build-tools')), // Good enough check for now.
		])

		return adb.isFile() && aapt.isDirectory()
	} catch (err) {}

	return false
}

const isIDeviceInstallerInstalled = async () => {
	try {
		await exec('ideviceinstaller --help')
		return true
	} catch (err) {}

	return false
}

const isIOSDeployInstalled = async () => {
	try {
		await exec('ios-deploy --version')
		return true
	} catch (err) {}

	return false
}

const isCarthageInstalled = async () => {
	try {
		await exec('carthage version')
		return true
	} catch (err) {}

	return false
}

const isXCodeInstalled = async () => {
	try {
		await exec('xcodebuild -version')
		return true
	} catch (err) {}

	return false
}

const runDoctor = async (appType) => {
	switch (appType) {
		case 'android': {
			let [appiumInstalled, appiumRunning, androidHomeSet] = await Promise.all([
				isAppiumInstalled(),
				isAppiumRunning(),
				isAndroidHomeSet(),
			])
			return {
				appiumInstalled,
				appiumRunning,
				androidHomeSet,
			}
		}

		case 'ios': {
			let [
				appiumInstalled,
				appiumRunning,
				ideviceinstallerInstalled,
				iosdeployInstalled,
				carthageInstalled,
				xcodeInstalled,
			] = await Promise.all([
				isAppiumInstalled(),
				isAppiumRunning(),
				isIDeviceInstallerInstalled(),
				isIOSDeployInstalled(),
				isCarthageInstalled(),
				isXCodeInstalled(),
			])
			return {
				appiumInstalled,
				appiumRunning,
				ideviceinstallerInstalled,
				iosdeployInstalled,
				carthageInstalled,
				xcodeInstalled,
			}
		}

		default:
			throw new Error(`Unknown app type: ${appType}`)
	}
}

module.exports = {
	isAppiumInstalled,
	isAppiumRunning,

	isAndroidHomeSet,

	isIDeviceInstallerInstalled,
	isIOSDeployInstalled,
	isCarthageInstalled,
	isXCodeInstalled,

	runDoctor,
}
