// @format

const fs = require('fs')
const path = require('path')
const PkgReader = require('reiko-parser')
const tmp = require('tmp')
const { execFile, execFileSync } = require('child_process')
const adb = require('adbkit')
const client = adb.createClient()

const escapeRegExp = (s) => s.replace(/[-/\\^$*+?.()|[\]{}]/g, '\\$&')

const readPackage = async (packagePath, packageType) => {
	var reader = await PkgReader(packagePath, packageType, {
		withIcon: false,
	})
	return new Promise(function(resolve, reject) {
		reader.parse((err, pkgInfo) => {
			if (err) {
				reject(err)
			} else {
				resolve(pkgInfo)
			}
		})
	})
}

exports.getAndroidDevices = async () => {
	console.log('Finding Android devices')
	return await client.listDevices()
}

exports.getAndroidDeviceProps = async (deviceId) => {
	console.log(`Finding props of ${deviceId}`)
	return await client.getProperties(deviceId)
}

exports.getAndroidPackages = async (deviceId) => {
	console.log(`Finding apps in ${deviceId}`)
	return await client.getPackages(deviceId)
}

exports.launchAndroidPackage = async (deviceId, appId, clearCache = true) => {
	console.log(`Launching app ${appId} on ${deviceId}`)
	if (clearCache) {
		console.log(`Clear cache ${appId}`)
		await client.shell(deviceId, `pm clear ${appId}`).then(adb.util.readAll)
	}
	return await client.shell(deviceId, `monkey -p ${appId} -c android.intent.category.LAUNCHER 1`).then(adb.util.readAll)
}

exports.getAndroidPackageInfo = async (deviceId, appId) => {
	console.log(`Finding package information for ${appId}`)
	const packageDump = await client
		.shell(deviceId, 'pm list packages -3 -f')
		.then(adb.util.readAll)
		.then((buf) => buf.toString())

	let match = packageDump.match(new RegExp(`^package:(.*)=${escapeRegExp(appId)}$`, 'm'))
	if (match == null) {
		throw new Error('No Package Found')
	}

	const tmpfile = tmp.fileSync()
	let apkPath = tmpfile.name

	let transfer = await client.pull(deviceId, match[1])
	await new Promise((resolve, reject) => {
		let w = transfer.pipe(
			fs.createWriteStream('', {
				fd: tmpfile.fd,
			}),
		)
		w.once('finish', resolve)
		w.once('error', reject)
	})

	return await readPackage(apkPath, 'apk')
}

exports.getiOSDevices = async () => {
	console.log('Finding iOS devices')
	if (process.platform !== 'darwin') {
		return []
	}

	try {
		await new Promise((resolve, reject) => {
			execFile(`ideviceinstaller`, ['--help'], (err, stdout, stderr) => {
				if (err != null) {
					return reject(err)
				}

				resolve()
			})
		})
	} catch (err) {
		throw new Error(
			'ideviceinstaller is not installed. Please run `brew install ideviceinstaller` before using Barista.',
		)
	}

	let ids = (await Promise.all([
		new Promise((resolve, reject) => {
			execFile(`idevice_id`, ['-l'], (err, stdout, stderr) => {
				if (err) {
					return reject(err)
				}

				resolve(
					stdout
						.split('\n')
						.map((line) => line.trim())
						.filter((line) => line)
						.map((line) => ({
							id: line,
							itype: 'dev',
						})),
				)
			})
		}),
	])).reduce((acc, cur) => [...acc, ...cur], [])

	return ids
}

exports.getiOSPackages = async (deviceId) => {
	console.log('Finding iOS packages on the connected device')
	let apps = await new Promise((resolve, reject) => {
		execFile(
			'ideviceinstaller',
			['-l'],
			//['-u', deviceId, '-l'],
			(err, stdout, stderr) => {
				if (err != null) {
					return reject(err)
				}
				resolve(stdout)
			},
		)
	})

	apps = await Promise.all(
		apps.split('\n').map(async (line) => {
			let match = line.match(/^([^,]+),\s+"(.+)",\s+"(.+)"$/m)
			if (match != null) {
				return {
					type: 'ios',
					name: match[3],
					id: match[1],
					version: match[2],
				}
			}

			match = line.match(/^([^\s]+)\s-\s(.*)\s([^\s]+)$/)
			if (match != null) {
				return {
					type: 'ios',
					name: match[2],
					id: match[1],
					version: match[3],
				}
			}

			// console.warn(`Couldn't determine app data for line: ${line}`)
			return null
		}),
	)

	return apps.filter((app) => app != null)
}
